import * as cover from './cover.js'

export async function Get(eTrack) {
    return Narrow(await Query(eTrack), eTrack)
}

export async function Query(eTrack) {
    let u = new URL('https://musicbrainz.org/ws/2/recording')

    u.searchParams.set('query', LuceneQuery(eTrack))
    u.searchParams.set('fmt', 'json')

    let r = await fetch(u, {
        headers: {
            "User-Agent": "hiss Audio Player"
        }
    })
        .catch(console.error)

    if(!r || !r.ok) {
    	return []
    }
	let d = await r.json()
	return d.recordings
}

export function LuceneQuery(eTrack) {
    let query = {}

    query.title = eTrack.name
    // query.dur = parseInt(eTrack.duration * 1000)
    if(eTrack.artist != null) {
        query.artist = eTrack.artist
    }
    if(eTrack.album != null) {
        query.release = eTrack.album
    }
    if(eTrack.trackNumber != null) {
        query.number = eTrack.trackNumber
    }

    return Object.entries(query)
        .filter(([ ,v ]) => v != null)
        .map(([ k, v ]) => k + ':' + JSON.stringify(v) )
        .join(' AND ')
}

export function Narrow(aRecordings, eTrack) {
    if(aRecordings.length == 0) {
        return null
    }
    if(aRecordings.length == 1) {
        return aRecordings[1]
    }

    let s = aRecordings
    	.map(r => {
    		r.diff = Math.abs((r.length / 1000) - eTrack.duration)
			return r
    	})
        .sort((a, b) => a.diff - b.diff)

    return s[0]
}

export function NeedsPolyfill(eTrack) {
    return eTrack.artist == null ||
        eTrack.coverId == null
}

export async function Polyfill(eTrack, recording) {
    let artistCredit = recording['artist-credit']
    
    if(eTrack.artist == null && artistCredit != null) {
        eTrack.artist = artistCredit[0].name
    }
	let release = MostRecentRelease(recording)
    if(eTrack.album == null) {
    	eTrack.artist = release.title
    }
    if(eTrack.coverId == null) {
        eTrack.coverId = await cover.IndexRelease(release)
    }
}

export function MostRecentRelease(recording) {
    let sr = recording.releases.sort((a, b) => new Date(b.date) - new Date(a.date))
    return sr[0]
}
